/************************* MPEG-2 NBC Audio Decoder **************************
 *                                                                           *
"This software module was originally developed in the course of
development of the MPEG-2 NBC/MPEG-4 Audio standard ISO/IEC 13818-7,
14496-1,2 and 3. This software module is an implementation of a part of one or more
MPEG-2 NBC/MPEG-4 Audio tools as specified by the MPEG-2 NBC/MPEG-4
Audio standard. ISO/IEC  gives users of the MPEG-2 NBC/MPEG-4 Audio
standards free license to this software module or modifications thereof for use in
hardware or software products claiming conformance to the MPEG-2 NBC/MPEG-4
Audio  standards. Those intending to use this software module in hardware or
software products are advised that this use may infringe existing patents.
The original developer of this software module and his/her company, the subsequent
editors and their companies, and ISO/IEC have no liability for use of this software
module or modifications thereof in an implementation. Copyright is not released for
non MPEG-2 NBC/MPEG-4 Audio conforming products.The original developer
retains full right to use the code for his/her  own purpose, assign or donate the
code to a third party and to inhibit third party from using the code for non
MPEG-2 NBC/MPEG-4 Audio conforming products. This copyright notice must
be included in all copies or derivative works."
Copyright(c)1996.
 *                                                                           *
 ****************************************************************************/
/*
 * $Id: channels.c,v 1.5 2001/09/04 18:39:35 menno Exp $
 */

#include "channels.h"
#include "coder.h"
#include "util.h"

static void addSCE(ChannelInfo *channelInfo, int numChannels, int numChannelsLeft, int tag);
static void addCPE(ChannelInfo *channelInfo, int numChannels, int numChannelsLeft, int tag);
static void addLFE(ChannelInfo *channelInfo, int numChannels, int numChannelsLeft, int tag);


void GetChannelInfo(ChannelInfo *channelInfo, int numFrontChannels, int numSideChannels, int numBackChannels, int numLFEChannels)
{
    int i;
    int sceTag           = 0;
    int cpeTag           = 0;
    int lfeTag           = 0;
    int numChannels      = (numFrontChannels +
                            numSideChannels  +
                            numBackChannels  +
                            numLFEChannels);
    int numChannelsLeft  = numChannels;

    /* Front channels */
    i = numFrontChannels;
    if (i % 2) {
        // Front Center
        addSCE(channelInfo, numChannels, numChannelsLeft, sceTag);
        numChannelsLeft--;
        sceTag++;
        i--;
    }
    while (i) {
        // Front Left/Right, Front Left/Right of Center, ???
        addCPE(channelInfo, numChannels, numChannelsLeft, cpeTag);
        numChannelsLeft -= 2;
        cpeTag++;
        i -= 2;
    }

    /* Side channels */
    i = numSideChannels;
    while (i > 1) {
        // Surround Left/Right (if rear surrounds present), ???
        addCPE(channelInfo, numChannels, numChannelsLeft, cpeTag);
        numChannelsLeft -= 2;
        cpeTag++;
        i -= 2;
    }
    if (i) {
        // ???
        addSCE(channelInfo, numChannels, numChannelsLeft, sceTag);
        numChannelsLeft--;
        sceTag++;
        i--;
    }

    /* Back channels */
    i = numBackChannels;
    while (i > 1) {
        // Surround Left/Right (if rear surrounds absent), Rear Surround Left/Right, ???
        addCPE(channelInfo, numChannels, numChannelsLeft, cpeTag);
        numChannelsLeft -= 2;
        cpeTag++;
        i -= 2;
    }
    if (i) {
        // Surround Center
        addSCE(channelInfo, numChannels, numChannelsLeft, sceTag);
        numChannelsLeft--;
        sceTag++;
        i--;
    }

    /* LFE channel */
    i = numLFEChannels;
    while (i) {
        addLFE(channelInfo, numChannels, numChannelsLeft, lfeTag);
        numChannelsLeft--;
        lfeTag++;
        i--;
    }
}

static void addSCE(ChannelInfo *channelInfo, int numChannels, int numChannelsLeft, int tag) {
    channelInfo[numChannels-numChannelsLeft].present         = 1;
    channelInfo[numChannels-numChannelsLeft].cpe             = 0;
    channelInfo[numChannels-numChannelsLeft].lfe             = 0;
    channelInfo[numChannels-numChannelsLeft].tag             = tag;
}

static void addCPE(ChannelInfo *channelInfo, int numChannels, int numChannelsLeft, int tag) {
    /* Left channel info */
    channelInfo[numChannels-numChannelsLeft].present         = 1;
    channelInfo[numChannels-numChannelsLeft].cpe             = 1;
    channelInfo[numChannels-numChannelsLeft].lfe             = 0;
    channelInfo[numChannels-numChannelsLeft].tag             = tag;
    channelInfo[numChannels-numChannelsLeft].ch_is_left      = 1;
    channelInfo[numChannels-numChannelsLeft].common_window   = 0;
    channelInfo[numChannels-numChannelsLeft].paired_ch       = numChannels-numChannelsLeft+1;
    /* Right channel info */
    channelInfo[numChannels-numChannelsLeft+1].present       = 1;
    channelInfo[numChannels-numChannelsLeft+1].cpe           = 1;
    channelInfo[numChannels-numChannelsLeft+1].lfe           = 0;
    channelInfo[numChannels-numChannelsLeft+1].ch_is_left    = 0;
    channelInfo[numChannels-numChannelsLeft+1].common_window = 0;
    channelInfo[numChannels-numChannelsLeft+1].paired_ch     = numChannels-numChannelsLeft;
}

static void addLFE(ChannelInfo *channelInfo, int numChannels, int numChannelsLeft, int tag) {
    channelInfo[numChannels-numChannelsLeft].present         = 1;
    channelInfo[numChannels-numChannelsLeft].cpe             = 0;
    channelInfo[numChannels-numChannelsLeft].lfe             = 1;
    channelInfo[numChannels-numChannelsLeft].tag             = tag;
}
